/* @flow */
/**
 *  Copyright (c) 2015, Facebook, Inc.
 *  All rights reserved.
 *
 *  This source code is licensed under the BSD-style license found in the
 *  LICENSE file in the root directory of this source tree. An additional grant
 *  of patent rights can be found in the PATENTS file in the same directory.
 */

'use strict';

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

exports['default'] = ArgumentsOfCorrectType;

var _error = require('../../error');

var _languagePrinter = require('../../language/printer');

var _typeDefinition = require('../../type/definition');

var _utilsKeyMap = require('../../utils/keyMap');

var _utilsKeyMap2 = _interopRequireDefault(_utilsKeyMap);

var _utilsIsValidLiteralValue = require('../../utils/isValidLiteralValue');

var _utilsIsValidLiteralValue2 = _interopRequireDefault(_utilsIsValidLiteralValue);

var _errors = require('../errors');

/**
 * Argument values of correct type
 *
 * A GraphQL document is only valid if all field argument literal values are
 * of the type expected by their position.
 */

function ArgumentsOfCorrectType(context) {
  return {
    Field: function Field(fieldAST) {
      var fieldDef = context.getFieldDef();
      if (!fieldDef) {
        return false;
      }
      var errors = [];
      var argASTs = fieldAST.arguments || [];

      var argASTMap = (0, _utilsKeyMap2['default'])(argASTs, function (arg) {
        return arg.name.value;
      });
      fieldDef.args.forEach(function (argDef) {
        var argAST = argASTMap[argDef.name];
        if (!argAST && argDef.type instanceof _typeDefinition.GraphQLNonNull) {
          errors.push(new _error.GraphQLError((0, _errors.missingArgMessage)(fieldAST.name.value, argDef.name, argDef.type), [fieldAST]));
        }
      });

      var argDefMap = (0, _utilsKeyMap2['default'])(fieldDef.args, function (def) {
        return def.name;
      });
      argASTs.forEach(function (argAST) {
        var argDef = argDefMap[argAST.name.value];
        if (argDef && !(0, _utilsIsValidLiteralValue2['default'])(argAST.value, argDef.type)) {
          errors.push(new _error.GraphQLError((0, _errors.badValueMessage)(argAST.name.value, argDef.type, (0, _languagePrinter.print)(argAST.value)), [argAST.value]));
        }
      });

      if (errors.length > 0) {
        return errors;
      }
    }
  };
}

module.exports = exports['default'];